function [ boolean ] = categorized_timings( p_nemo_data, p_config )
%TIMING_INFORMATION Plots clock details.
%
%   Returns:
%   boolean: False if something went wrong during the plotting process.
%
%   Parameters:
%   p_nemo_data:      Struct, containing all float data (load with nemo.load.data()).
%   p_config:         Global configuration file. Load using nemo.load.config().

%% Initialize return variable
boolean = false;

%% Parameter check
parameter_error = false;

if (~iscell(p_nemo_data) || ~isstruct(p_config))
    parameter_error = true;
end

if (parameter_error) % display error message
    disp([ mfilename ': Wrong parameter! Please check your input parameters carefully!']);
    return;
else % do some cleanup
    clear parameter_error;
end

%% Setup directories
directory = fullfile(p_config.GENERAL.storage_directory_plots, p_config.PLOT_CATEGORIZED_TIMINGS.storage_directory_image);
directory_fig = fullfile(p_config.GENERAL.storage_directory_plots, p_config.PLOT_CATEGORIZED_TIMINGS.storage_directory_fig);
image_type = p_config.PLOT_CATEGORIZED_TIMINGS.image_type;
if ~exist(directory, 'dir')
    system(['mkdir ' directory]);
end
if ~exist(directory_fig, 'dir')
    mkdir(directory_fig);
end

%% Process every float

float_count = length(p_nemo_data);

for o_float = 1:float_count
    
    current_float = p_nemo_data{o_float};
    
    start_date = current_float{end}.OVERALL_MISSION_INFORMATION.start_date;
    start_time = current_float{end}.OVERALL_MISSION_INFORMATION.start_time;
    xmit_ascent_start_time = nemo.float.collect(current_float, 'PROFILE_TECHNICAL_DATA', 'xmit_ascent_start_time');
    xmit_ascent_end_time = nemo.float.collect(current_float, 'PROFILE_TECHNICAL_DATA', 'xmit_ascent_end_time');
    xmit_descent_start_time = nemo.float.collect(current_float, 'PROFILE_TECHNICAL_DATA', 'xmit_descent_start_time');
    xmit_parking_start_time = nemo.float.collect(current_float, 'PROFILE_TECHNICAL_DATA', 'xmit_parking_start_time');
    xmit_upcast_start_time = nemo.float.collect(current_float, 'PROFILE_TECHNICAL_DATA', 'xmit_upcast_start_time');
    xmit_surface_start_time = nemo.float.collect(current_float, 'PROFILE_TECHNICAL_DATA', 'xmit_surface_start_time');
    xmit_profile_number = nemo.float.collect(current_float, 'PROFILE_TECHNICAL_DATA', 'xmit_profile_number');
    
    combined_xmit_times = [ ...
        xmit_descent_start_time, ...
        xmit_parking_start_time, ...
        xmit_upcast_start_time, ...
        xmit_ascent_start_time, ...
        xmit_ascent_end_time, ...
        xmit_surface_start_time ...
        ];
    
    combined_diff_times = [ ...
%         NaN(length(xmit_descent_start_time), 1), ...
        (xmit_descent_start_time - xmit_parking_start_time), ...
        (xmit_parking_start_time - xmit_upcast_start_time), ...
        (xmit_upcast_start_time - xmit_ascent_start_time), ...
        (xmit_ascent_start_time - xmit_ascent_end_time), ...
        (xmit_ascent_end_time - xmit_surface_start_time), ...
        [NaN; (xmit_surface_start_time(1:end -1) - xmit_descent_start_time(2:end))]
        ];

    [p_row, ~] = size(combined_xmit_times);
    tmp_xmit_times = [];
    tmp_diff_times = [];
    for o_row = 1:p_row
        tmp_xmit_times = [ tmp_xmit_times; combined_xmit_times(o_row, :)'];
        tmp_diff_times = [ tmp_diff_times; combined_diff_times(o_row, :)'];
    end
    % categorize them
    combined_xmit_times = [repmat([3:6, 2, 1]', length(tmp_xmit_times)/6, 1), tmp_xmit_times];
    combined_diff_times = [repmat([3:6, 2, 1]', length(tmp_diff_times)/6, 1), tmp_diff_times];
    clear tmp_xmit_times tmp_diff_times
    
    
    % get plot details
    width = p_config.PLOT_CATEGORIZED_TIMINGS.image_width;
    height = p_config.PLOT_CATEGORIZED_TIMINGS.image_height;
    paperunit = p_config.PLOT_CATEGORIZED_TIMINGS.image_paperunit;
    
    % plot collected data
    
    figure_id = figure('Visible', 'off'); % create figure
    % initialize figure
    set(figure_id,'InvertHardcopy','on');
    set(figure_id,'PaperUnits', paperunit);
    papersize = get(figure_id, 'PaperSize');
    left = (papersize(1)- width)/2;
    bottom = (papersize(2)- height)/2;
    myfiguresize = [left, bottom, width, height];
    set(figure_id,'PaperPosition', myfiguresize);
    
    clear height width left bottom papersize myfiguresize
    
    %set(gcf, 'Position', get(0,'ScreenSize'), 'color', [1 1 1]);
    sub = {};
    
    sub{1} = subplot(2,1,1);
    hold on


    plot(combined_xmit_times(:, 2), combined_xmit_times(:, 1), ...
         'color', [0 0 0], ...
         'linestyle', '-', ...
         'linewidth', 1 ...
     );
     scatter(combined_xmit_times(combined_xmit_times(:, 1) == 3, 2), combined_xmit_times(combined_xmit_times(:, 1) == 3, 1), ...
         'marker', 'o', ...
         'MarkerFaceColor', [1 0 0], ...
         'MarkerEdgeColor', [1 0 0] ...
         );
     scatter(combined_xmit_times(combined_xmit_times(:, 1) == 4, 2), combined_xmit_times(combined_xmit_times(:, 1) == 4, 1), ...
         'marker', 'o', ...
         'MarkerFaceColor', [0 1 0], ...
         'MarkerEdgeColor', [0 1 0] ...
         );
     scatter(combined_xmit_times(combined_xmit_times(:, 1) == 5, 2), combined_xmit_times(combined_xmit_times(:, 1) == 5, 1), ...
         'marker', 'o', ...
         'MarkerFaceColor', [0 0 1], ...
         'MarkerEdgeColor', [0 0 1] ...
         );
     scatter(combined_xmit_times(combined_xmit_times(:, 1) == 6, 2), combined_xmit_times(combined_xmit_times(:, 1) == 6, 1), ...
         'marker', 'o', ...
         'MarkerFaceColor', [1 1 0], ...
         'MarkerEdgeColor', [1 1 0] ...
         );
     scatter(combined_xmit_times(combined_xmit_times(:, 1) == 2, 2), combined_xmit_times(combined_xmit_times(:, 1) == 2, 1), ...
         'marker', 'o', ...
         'MarkerFaceColor', [0 1 1], ...
         'MarkerEdgeColor', [0 1 1] ...
         );
     scatter(combined_xmit_times(combined_xmit_times(:, 1) == 1, 2), combined_xmit_times(combined_xmit_times(:, 1) == 1, 1), ...
         'marker', 'o', ...
         'MarkerFaceColor', [1 0 1], ...
         'MarkerEdgeColor', [1 0 1] ...
         );

    ylabel('Category');
    xlabel('Seconds since start');
    axis ij;
%     legend('SurfaceStart-GPS', 'SurfaceStart-IRIDIUM', 'AscentStart-RTC2000');
    leg1 = legend('TimesConnected', 'DescentStart', 'ParkingStart', 'UpcastStart', 'AscentStart', 'AscentEnd', 'SurfaceStart');
    set(leg1, 'Location', 'eastoutside', 'Orientation', 'vertical');
    clear leg1
    t = title(...
        [ 'Float ' ...
          num2str(p_nemo_data{o_float}{1}.FLOAT_IDENTIFICATION.float_serial_number) ...
          ' @ ' ...
          datestr(datetime('now', 'Format', 'dd-MMM-yyyy HH:mm:SS')) ...
        ]);
    set(t, 'FontSize', 18);
    clear t
    grid on
    box on
    hold off
    
    sub{2} = subplot(2,1,2);
    hold on


    plot(combined_xmit_times(:, 2), combined_diff_times(:, 2), ...
         'color', [0 0 0], ...
         'linestyle', '-', ...
         'linewidth', 1 ...
     );
     scatter(combined_xmit_times(combined_xmit_times(:, 1) == 3, 2), combined_diff_times(combined_diff_times(:, 1) == 3, 2), ...
         'marker', 'o', ...
         'MarkerEdgeColor', [1 0 0] ...
     );
    scatter(combined_xmit_times(combined_xmit_times(:, 1) == 4, 2), combined_diff_times(combined_diff_times(:, 1) == 4, 2), ...
         'marker', 'o', ...
         'MarkerEdgeColor', [0 1 0] ...
     );
    scatter(combined_xmit_times(combined_xmit_times(:, 1) == 5, 2), combined_diff_times(combined_diff_times(:, 1) == 5, 2), ...
         'marker', 'o', ...
         'MarkerEdgeColor', [0 0 1] ...
     );
    scatter(combined_xmit_times(combined_xmit_times(:, 1) == 6, 2), combined_diff_times(combined_diff_times(:, 1) == 6, 2), ...
         'marker', 'o', ...
         'MarkerEdgeColor', [1 1 0] ...
     );
    scatter(combined_xmit_times(combined_xmit_times(:, 1) == 2, 2), combined_diff_times(combined_diff_times(:, 1) == 2, 2), ...    
         'marker', 'o', ...
         'MarkerEdgeColor', [0 1 1] ...
     );
    scatter(combined_xmit_times(combined_xmit_times(:, 1) == 1, 2), combined_diff_times(combined_diff_times(:, 1) == 1, 2), ...
         'marker', 'o', ...
         'MarkerEdgeColor', [1 0 1] ...
     );
 
 
    % plot all times > 0 and hide them from legend
    selector = combined_diff_times(:, 1) == 3 & combined_diff_times(:, 2) > 0;
    scat1 = scatter(combined_xmit_times(selector, 2), combined_diff_times(selector, 2), ...
         'marker', 'o', ...
         'MarkerFaceColor', [1 0 0], ...
         'MarkerEdgeColor', [1 0 0] ...
     );
    hasbehavior(scat1, 'legend', false);
    selector = combined_diff_times(:, 1) == 4 & combined_diff_times(:, 2) > 0;
    scat1 = scatter(combined_xmit_times(selector, 2), combined_diff_times(selector, 2), ...
         'marker', 'o', ...
         'MarkerFaceColor', [0 1 0], ...
         'MarkerEdgeColor', [0 1 0] ...
     );
    hasbehavior(scat1, 'legend', false);
    selector = combined_diff_times(:, 1) == 5 & combined_diff_times(:, 2) > 0;
    scat1 = scatter(combined_xmit_times(selector, 2), combined_diff_times(selector, 2), ...
         'marker', 'o', ...
         'MarkerFaceColor', [0 0 1], ...
         'MarkerEdgeColor', [0 0 1] ...
     );
    hasbehavior(scat1, 'legend', false);
    selector = combined_diff_times(:, 1) == 6 & combined_diff_times(:, 2) > 0;
    scat1 = scatter(combined_xmit_times(selector, 2), combined_diff_times(selector, 2), ...
         'marker', 'o', ...
         'MarkerFaceColor', [1 1 0], ...
         'MarkerEdgeColor', [1 1 0] ...
     );
    hasbehavior(scat1, 'legend', false);
    selector = combined_diff_times(:, 1) == 2 & combined_diff_times(:, 2) > 0;
    scat1 = scatter(combined_xmit_times(selector, 2), combined_diff_times(selector, 2), ...    
         'marker', 'o', ...
         'MarkerFaceColor', [0 1 1], ...
         'MarkerEdgeColor', [0 1 1] ...
     );
    hasbehavior(scat1, 'legend', false);
    selector = combined_diff_times(:, 1) == 1 & combined_diff_times(:, 2) > 0;
    scat1 = scatter(combined_xmit_times(selector, 2), combined_diff_times(selector, 2), ...
         'marker', 'o', ...
         'MarkerFaceColor', [1 0 1], ...
         'MarkerEdgeColor', [1 0 1] ...
     );
    hasbehavior(scat1, 'legend', false);
    clear scat1
 
 
    ylabel('Time difference [sec]');
    xlabel('Seconds since start');
%     axis ij;
%     legend('SurfaceStart-GPS', 'SurfaceStart-IRIDIUM', 'AscentStart-RTC2000');
    leg1 = legend('DiffsConnected', 'DescentStart-ParkingStart', 'ParkingStart-UpcastStart', 'UpcastStart-AscentStart', 'AscentStart-AscentEnd', 'AscentEnd-SurfaceStart', 'SurfaceStart-DescentStart');
    set(leg1, 'Location', 'eastoutside', 'Orientation', 'vertical');
    clear leg1
    grid on
    box on
    hold off
    
    sub1_pos = get(sub{1}, 'Position');
    sub2_pos = get(sub{2}, 'Position');
%     leg1_pos = get(leg1, 'Position');
    set(sub{1}, 'Position', [sub1_pos(1:2) 0.6652 sub1_pos(4)]);
    set(sub{2}, 'Position', [sub2_pos(1:2) 0.6652 sub2_pos(4)]);
    
    % store plot
    
    image_filename = [p_config.PLOT_CATEGORIZED_TIMINGS.image_filename ...
        num2str(p_nemo_data{o_float}{1}.FLOAT_IDENTIFICATION.float_serial_number)];
    figure_filename = [p_config.PLOT_CATEGORIZED_TIMINGS.figure_filename ...
        num2str(p_nemo_data{o_float}{1}.FLOAT_IDENTIFICATION.float_serial_number)];
    
    if (p_config.PLOT_CATEGORIZED_TIMINGS.save_figure)
        set(gcf,'CreateFcn','set(gcf,''Visible'',''on'')');
        saveas(figure_id, ...
               [directory_fig figure_filename]);
     end
    
    
    print(figure_id, [directory image_filename], image_type);
    
    close(figure_id);

end

boolean = true;


    function current_plot = set_x_axis(current_plot)
        
        best_guess_datevec = nemo.float.collect(current_float, 'SECOND_ORDER_INFORMATION', 'best_guess_datevec');
        % get min and max datevec
        min_datenum = min(datenum(best_guess_datevec));
        max_datenum = max(datenum(best_guess_datevec));
        if (min_datenum == max_datenum)
            max_datenum = max_datenum + 1;
        end
        diff_datenum = max_datenum - min_datenum;
        if (diff_datenum > 365)
            x_tick = [min_datenum:365:max_datenum];
        elseif (diff_datenum > 30)
            x_tick = [min_datenum:30:max_datenum];
        elseif (diff_datenum > 5)
            x_tick = [min_datenum:5:max_datenum];
        else
            x_tick = [min_datenum:1:max_datenum];
        end
        if (x_tick(end) < max_datenum)
            x_tick = [ x_tick max_datenum ];
        end
        x_tick_labels = cellstr(datestr(x_tick, 'yyyy-mm-dd'));
        
        if (~isempty(x_tick_labels))
            x_tick_labels{end} = '';
        end
        
        set(current_plot, 'XLimMode', 'manual', ...
            'XLim', [min_datenum, max_datenum], ...
            'XTick', x_tick, ...
            'XTickLabel', x_tick_labels);
        
    end

end

